<?php
namespace WPCommander;

defined('ABSPATH') || exit;

class App extends Singleton {
    private $ajax = [
        'get_template' => 'wpcommander_get_template',
        'get_app_data' => 'wpcommander_get_app_data',
        'update_app_data' => 'wpcommander_update_app_data',
        'get_settings_data' => 'wpcommander_get_settings_data',
        'update_settings_data' => 'wpcommander_update_settings_data'
    ];
    private $access = false;
    private $cache_key_update = 'wpcommander_update';

    public static function activate() {
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();

        $table = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
        $sql = "CREATE TABLE {$table} (
			id bigint UNSIGNED NOT NULL AUTO_INCREMENT,
			title varchar(255) DEFAULT NULL,
	        parent bigint UNSIGNED NOT NULL DEFAULT 0,
	        color varchar(7) DEFAULT NULL,
			user_id bigint UNSIGNED NOT NULL DEFAULT 0,
			count bigint UNSIGNED NOT NULL DEFAULT 0,
			sort bigint UNSIGNED NOT NULL DEFAULT 0,
			created datetime NOT NULL,
			modified datetime NOT NULL,
			PRIMARY KEY id (id)
		) {$charset_collate};";
        dbDelta($sql);

        $table = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';
        $sql = "CREATE TABLE {$table} (
            folder_id bigint UNSIGNED NOT NULL,
            attachment_id bigint UNSIGNED NOT NULL,
            PRIMARY KEY id (folder_id,attachment_id)
        ) {$charset_collate};";
        dbDelta($sql);

        App::activate_db();
        App::activate_options();
    }

    public static function activate_db() {
        $key = 'wpcommander_state';
        $options = App::get_options($key);
        if($options == NULL) {
        }
    }

    public static function activate_options() {
        $key = 'wpcommander_state';
        $options = App::get_options($key);
        if($options == NULL) {
            $options = [];
            $options['first_activation'] = time();
        }
        $options['plugin_version'] = WPCOMMANDER_PLUGIN_VERSION;
        $options['db_version'] = WPCOMMANDER_PLUGIN_DB_VERSION;
        App::set_options($key, $options);

        $key = 'wpcommander_settings';
        $options = App::get_options($key);
        if($options == NULL) {
            $options = [
                'roles' => ['administrator'],
                'default_color' => NULL,
                'disable_counter' => false,
                'disable_ajax' => false,
                'infinite_scrolling' => false
            ];
            App::set_options($key, $options);
        }
    }

    public static function deactivate() {
    }

    public static function get_options($key) {
        $option_value = get_option($key);
        if($option_value) {
            return unserialize($option_value);
        }
        return NULL;
    }

    public static function set_options($key, $options) {
        $option_value = serialize($options);

        if(get_option($key) == false) {
            $deprecated = null;
            $autoload = 'no';
            return add_option($key, $option_value, $deprecated, $autoload);
        } else {
            $old_value = get_option($key);
            if($old_value === $option_value) {
                return true;
            } else {
                return update_option($key, $option_value);
            }
        }
    }

    public function initialization() {
        add_action('init', array($this, 'init'));

        add_action('after_plugin_row', [$this, 'after_plugin_row'], 10, 2);
        add_filter('wp_get_update_data', [$this, 'get_update_data'], 10, 2 );
    }

    private function request_update_data() {
        $data = get_transient($this->cache_key_update);

        if(false === $data) {
            $data = wp_remote_get(WPCOMMANDER_UPDATE_URL . '?plugin=' . WPCOMMANDER_PLUGIN_NAME, ['timeout' => 10, 'headers' => ['Accept' => 'application/json']]);
            if(is_wp_error($data) || 200 !== wp_remote_retrieve_response_code($data) || empty( wp_remote_retrieve_body($data))) {
                return false;
            }
            set_transient($this->cache_key_update, $data, DAY_IN_SECONDS);
        }

        $data = json_decode(wp_remote_retrieve_body($data));
        return $data;
    }

    private function need_to_update() {
        $data = $this->request_update_data();
        if($data && version_compare( WPCOMMANDER_PLUGIN_VERSION, $data->version, '<')) {
            return true;
        }
        return false;
    }

    public function after_plugin_row($plugin_file, $plugin_data) {
        if(WPCOMMANDER_PLUGIN_BASE_NAME !== $plugin_file || !$this->need_to_update()) {
            return;
        }

        $update_data = $this->request_update_data();

        global $wp_list_table;
        $columns_count = $wp_list_table->get_column_count();
        ?>
        <tr class='plugin-update-tr active' data-plugin='<?php echo WPCOMMANDER_PLUGIN_BASE_NAME; ?>'>
            <td colspan='<?php echo $columns_count; ?>' class='plugin-update'>
                <div class='update-message notice inline notice-warning notice-alt'>
                    <p>
                        <?php echo sprintf(esc_html__('There is a new version %1s of WPCommander available.', 'wpcommander'), $update_data->version) ?>
                        <a href="<?php echo WPCOMMANDER_PLUGIN_SITE_URL; ?>"><?php esc_html_e('Update now.', 'wpcommander') ?></a>
                    </p>
                </div>
            </td>
        </tr>
        <?php
    }

    public function get_update_data($update_data, $titles) {
        if($this->need_to_update()) {
            $update_data['counts']['plugins'] = $update_data['counts']['plugins']  + 1;
        }
        return $update_data;
    }

    public function init() {
        $config = $this->get_config_data();
        if(isset($config->infinite_scrolling) && $config->infinite_scrolling) {
            add_filter('media_library_infinite_scrolling', '__return_true');
        }

        add_action('admin_notices', [$this, 'admin_notices']);
        add_action('admin_head', [$this, 'admin_head']);
        add_action('admin_menu', [$this, 'admin_menu']);
        add_action('admin_footer', [$this, 'admin_footer']);
        add_action('admin_enqueue_scripts', [$this, 'enqueue_scripts']);
        add_action('wp_enqueue_scripts', [$this, 'enqueue_scripts']);

        add_action('in_admin_header', [$this, 'in_admin_header']);
        add_action('delete_post', [$this, 'delete_post']);
        add_action('add_attachment', [$this, 'add_attachment']);
        add_action('pre_get_posts', [$this, 'redirect_to_main']);

        add_action('wp_ajax_' . $this->ajax['get_template'], [$this, 'ajax_get_template']);
        add_action('wp_ajax_' . $this->ajax['get_app_data'], [$this, 'ajax_get_app_data']);
        add_action('wp_ajax_' . $this->ajax['update_app_data'], [$this, 'ajax_update_app_data']);
        add_action('wp_ajax_' . $this->ajax['get_settings_data'], [$this, 'ajax_get_settings_data']);
        add_action('wp_ajax_' . $this->ajax['update_settings_data'], [$this, 'ajax_update_settings_data']);

        add_filter('plugin_action_links_' . WPCOMMANDER_PLUGIN_BASE_NAME, [$this, 'plugin_action_links'], 10, 4);
        add_filter('posts_clauses', [$this, 'posts_clauses']);

        $this->register_gutenberg_blocks();
    }

    public function register_gutenberg_blocks() {
    }

    public function plugin_action_links($actions) {
        $links = [];
        $links['settings'] = '<a href="' . network_admin_url('options-general.php?page=wpcommander-settings') . '">Settings</a>';
        return array_merge($actions, $links);
    }

    public function admin_notices() {
        if(get_option('wpcommander_dismiss_first_use_notification', false) || (get_current_screen() && get_current_screen()->base === 'upload')) {
            return;
        }

        $classes = [
            'notice',
            'notice-info',
            'is-dismissible',
        ];
        $msg = sprintf(
            '<span>' . esc_html__('Thanks for start using the WPCommander plugin. Let\'s create first folders. %s', 'wpcommander') . '</span>',
            '<a href="' . esc_url(admin_url('/upload.php')) . '">' . esc_html__('Go to WordPress Media Library.', 'wpcommander') . '</a>'
        );
        printf('<div id="wpcommander-first-use-notification" class="%s"><p>%s</p></div>', trim(implode(' ',$classes)), $msg);
    }

    public function admin_head() {
        echo '<style>#screen-meta-links {position: absolute; right: 0;} .wrap {margin-top: 15px;}</style>';
    }

    public function admin_menu() {
        $slug = 'wpcommander-settings';

        add_submenu_page(
            'options-general.php',
            'WPCommander Settings',
            'WPCommander',
            'manage_options',
            $slug,
            [$this, 'admin_menu_page_settings']
        );
    }

    public function admin_footer() {
        if(get_current_screen() && get_current_screen()->base !== 'plugins') {
            return;
        }

        $globals = [
            'token' => $this->get_token(),
            'ajax' => [
                'url' => WPCOMMANDER_FEEDBACK_URL
            ]
        ];

        wp_enqueue_style('wpcommander-feedback', WPCOMMANDER_PLUGIN_URL . 'assets/css/feedback.css', [], WPCOMMANDER_PLUGIN_VERSION);
        wp_enqueue_script('wpcommander-feedback', WPCOMMANDER_PLUGIN_URL . 'assets/js/feedback.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false);
        wp_localize_script('wpcommander-feedback', 'wpcommander_feedback_globals', $globals);

        require_once(plugin_dir_path(dirname(__FILE__)) . 'templates/feedback.php');
    }

    private function get_config_data() {
        $config = $this->get_options('wpcommander_settings');
        if($config) {
            return (object) $config;
        }
        return (object) [
            'roles' => [],
            'default_color' => NULL,
            'disable_counter' => false,
            'disable_ajax' => false,
            'infinite_scrolling' => false
        ];
    }

    private function set_config_data($config) {
        return $this->set_options('wpcommander_settings', (array) $config);
    }

    private function get_messages($extra = false): array {
        $messages = [
            'success' => esc_html__('The operation completed successfully', 'wpcommander'),
            'failed' => esc_html__('The operation failed', 'wpcommander')
        ];
        if($extra) {
            $messages['status'] = [
                '400' => esc_html__('The license is already registered for other domain', 'wpcommander'),
                '401' => esc_html__('Failed to connect to license server', 'wpcommander'),
                '402' => esc_html__('The personal token is invalid or has been deleted', 'wpcommander'),
                '403' => esc_html__('The personal token is missing the required permission for license server', 'wpcommander'),
                '404' => esc_html__('Invalid purchase code', 'wpcommander'),
                '405' => esc_html__('Something goes wrong, try again shortly', 'wpcommander'),
                '406' => esc_html__('Parsing error, try again shortly', 'wpcommander'),
                '407' => esc_html__('Error updating license data', 'wpcommander'),
            ];
        }

        return $messages;
    }

    private function get_admin_globals(): array {
        $globals = [ // nested array supports boolean type
            'data' => [
                'plugin' => WPCOMMANDER_PLUGIN_BASE_NAME,
                'update' => $this->need_to_update(),
                'version' => WPCOMMANDER_PLUGIN_VERSION
            ],
            'ajax' => [
                'url' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('wpcommander-ajax-nonce'),
                'actions' => [
                    'get_data' => $this->ajax['get_settings_data'],
                    'update_data' => $this->ajax['update_settings_data']
                ]
            ]
        ];
        return $globals;
    }

    private function get_globals(): array {
        $post_type = $this->get_post_type();
        $config = $this->get_config_data();

        $globals = [ // nested array supports boolean type
            'data' => [
                'version' => WPCOMMANDER_PLUGIN_VERSION,
                'meta' => $this->get_meta($post_type),
                'default_color' => $config->default_color,
                'disable_counter' => $config->disable_counter,
                'disable_ajax' => $config->disable_ajax,
                'type' => $post_type
            ],
            'msg' => $this->get_messages(),
            'ajax' => [
                'url' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('wpcommander-ajax-nonce'),
                'actions' => [
                    'get_template' => $this->ajax['get_template'],
                    'get_data' => $this->ajax['get_app_data'],
                    'update_data' => $this->ajax['update_app_data']
                ]
            ]
        ];
        return $globals;
    }

    private function get_meta($type) {
        $user_id = get_current_user_id();
        $meta_default = [
            'folder' => -1, // All items
            'collapsed' => null,
            'sort' => [
                'items' => null
            ]
        ];
        $user_meta = (array)get_user_meta($user_id, 'wpcommander_states', true);
        $meta = isset($user_meta['types'][$type]) ? array_replace_recursive($meta_default, $user_meta['types'][$type]) : $meta_default;
        return $meta;
    }

    private function get_post_type() {
        $type = 'attachment';
        return $type;
    }

    private function has_access(): bool {
        $config = $this->get_config_data();

        $user_id = get_current_user_id();
        $user = get_user_by('id', $user_id);
        $exist = false;
        if($user) {
            foreach($user->roles as $user_role) {
                foreach($config->roles as $role) {
                    if($role == $user_role) {
                        $exist = true;
                        break;
                    }
                }
            }
        }

        return $exist;
    }

    public function admin_menu_page_settings() {
        $page = filter_input(INPUT_GET, 'page', FILTER_DEFAULT);
        if($page === 'wpcommander-settings') {
            $globals = [
                'version' => WPCOMMANDER_PLUGIN_VERSION,
                'msg' => $this->get_messages(true),
                'ajax' => [
                    'url' => admin_url('admin-ajax.php'),
                    'nonce' => wp_create_nonce('wpcommander-ajax-nonce'),
                    'actions' => [
                        'get_template' => $this->ajax['get_template'],
                        'get_data' => $this->ajax['get_settings_data'],
                        'update_data' => $this->ajax['update_settings_data']
                    ]
                ]
            ];

            wp_enqueue_style('wpcommander-settings', WPCOMMANDER_PLUGIN_URL . 'assets/css/settings.css', [], WPCOMMANDER_PLUGIN_VERSION);
            wp_enqueue_style('wpcommander-notify', WPCOMMANDER_PLUGIN_URL . 'assets/css/notify.css', [], WPCOMMANDER_PLUGIN_VERSION);
            wp_enqueue_style('wpcommander-colorpicker', WPCOMMANDER_PLUGIN_URL . 'assets/css/colorpicker.css', [], WPCOMMANDER_PLUGIN_VERSION);
            wp_enqueue_script('wpcommander-cookies', WPCOMMANDER_PLUGIN_URL . 'assets/js/cookies.js', [], WPCOMMANDER_PLUGIN_VERSION, false);
            wp_enqueue_script('wpcommander-notify', WPCOMMANDER_PLUGIN_URL . 'assets/js/notify.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false);
            wp_enqueue_script('wpcommander-colorpicker', WPCOMMANDER_PLUGIN_URL . 'assets/js/colorpicker.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false);
            wp_enqueue_script('wpcommander-settings', WPCOMMANDER_PLUGIN_URL . 'assets/js/settings.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false );
            wp_enqueue_script('wpcommander-angular-light', WPCOMMANDER_PLUGIN_URL . 'assets/js/lib/angular-light/angular-light.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false );
            wp_localize_script('wpcommander-settings', 'wpcommander_settings_globals', $globals);

            require_once(plugin_dir_path( dirname(__FILE__) ) . 'templates/settings.php');
        }
    }

    public function enqueue_scripts() {
        wp_enqueue_style('wpcommander-admin', WPCOMMANDER_PLUGIN_URL . 'assets/css/admin.css', [], WPCOMMANDER_PLUGIN_VERSION);
        wp_enqueue_script('wpcommander-admin', WPCOMMANDER_PLUGIN_URL . 'assets/js/admin.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false);
        wp_localize_script('wpcommander-admin', 'wpcommander_admin_globals', $this->get_admin_globals());

        if($this->has_access()) {
            wp_enqueue_style('wpcommander-notify', WPCOMMANDER_PLUGIN_URL . 'assets/css/notify.css', [], WPCOMMANDER_PLUGIN_VERSION);
            wp_enqueue_style('wpcommander-colorpicker', WPCOMMANDER_PLUGIN_URL . 'assets/css/colorpicker.css', [], WPCOMMANDER_PLUGIN_VERSION);
            wp_enqueue_style('wpcommander-app', WPCOMMANDER_PLUGIN_URL . 'assets/css/app.css', [], WPCOMMANDER_PLUGIN_VERSION);
            wp_enqueue_script('wpcommander-cookies', WPCOMMANDER_PLUGIN_URL . 'assets/js/cookies.js', [], WPCOMMANDER_PLUGIN_VERSION, false);
            wp_enqueue_script('wpcommander-notify', WPCOMMANDER_PLUGIN_URL . 'assets/js/notify.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false);
            wp_enqueue_script('wpcommander-colorpicker', WPCOMMANDER_PLUGIN_URL . 'assets/js/colorpicker.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false);
            wp_enqueue_script('wpcommander-tree', WPCOMMANDER_PLUGIN_URL . 'assets/js/tree.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false);
            wp_enqueue_script('wpcommander-app', WPCOMMANDER_PLUGIN_URL . 'assets/js/app.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false);
            wp_localize_script('wpcommander-app', 'wpcommander_globals', $this->get_globals());
        }
    }

    public function in_admin_header() {
        $page = sanitize_key(filter_input(INPUT_GET, 'page', FILTER_DEFAULT));
        if($page === 'wpcommander-settings') {
            remove_all_actions('admin_notices');
            remove_all_actions('all_admin_notices');
            add_action('admin_notices', [$this, 'admin_notices']);
        }
    }

    public function delete_post($post_id) {
        global $wpdb;
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
        $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

        // folders to refresh after the update
        $sql = $wpdb->prepare("SELECT DISTINCT folder_id as id FROM {$tableAttachments} WHERE attachment_id=%d", $post_id);
        $folders = $wpdb->get_results($sql, 'ARRAY_A');

        $foldersToRefresh = [];
        foreach($folders as $folder) {
            $foldersToRefresh[] = $folder['id'];
        }

        // remove a post from the attachments table
        $sql = $wpdb->prepare("DELETE FROM {$tableAttachments} WHERE attachment_id=%d", $post_id);
        $wpdb->query($sql);

        // update the attachment count
        if(!empty($foldersToRefresh)) {
            $ids = implode(',', array_map('absint', $foldersToRefresh));
            $sql = $wpdb->prepare("UPDATE {$tableFolders} AS F SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id = F.id) WHERE id IN(%1s)", $ids);
            $wpdb->query($sql);
        }
    }

    public function add_attachment($attachment_id) {
        if (isset($_REQUEST['folder'])) {
            $folder_id = intval(sanitize_text_field($_REQUEST['folder']), 10);

            global $wpdb;
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
            $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

            // add new attachments
            $wpdb->insert(
                $tableAttachments,
                [
                    'folder_id' => $folder_id,
                    'attachment_id' => $attachment_id
                ]
            );

            // update the attachment count
            $sql = $wpdb->prepare("UPDATE {$tableFolders} AS F SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id = F.id) WHERE id=%d", $folder_id);
            $wpdb->query($sql);
        }
    }

    public function redirect_to_main() {
        $this->access = $this->has_access();
    }

    public function posts_clauses($clauses) {
        if($this->access && $this->get_post_type() && strpos($clauses['where'], $this->get_post_type()) !== false) {
            $mode = sanitize_key(filter_input(INPUT_POST, 'wpcommander_mode', FILTER_DEFAULT));
            if(empty($mode)) {
                return $clauses;
            }

            global $wpdb;
            $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

            $type = $this->get_post_type();
            $meta = $this->get_meta($type);
            $folder = $meta['folder'];

            if ($folder > 0) {
                $clauses['join'] .= " LEFT JOIN $tableAttachments AS ATTACHMENTS ON ($wpdb->posts.ID = ATTACHMENTS.attachment_id)";
                $clauses['where'] = " AND (ATTACHMENTS.folder_id = $folder) " . $clauses['where'];
            } else if ($folder == -2) { // uncategorized
                $user_id = get_current_user_id();
                $clauses['where'] .= " AND ($wpdb->posts.ID NOT IN (SELECT ATTACHMENTS.attachment_id FROM $tableAttachments AS ATTACHMENTS LEFT JOIN $tableFolders AS FOLDERS ON FOLDERS.id=ATTACHMENTS.folder_id WHERE FOLDERS.user_id=$user_id))";
            }

            switch($meta['sort']['items']) {
                case 'name-asc': {
                    $clauses['orderby'] = " $wpdb->posts.post_title ASC, " . $clauses['orderby'];
                } break;
                case 'name-desc': {
                    $clauses['orderby'] = " $wpdb->posts.post_title DESC, " . $clauses['orderby'];
                } break;
                case 'date-asc': {
                    $clauses['orderby'] = " $wpdb->posts.post_date ASC, " . $clauses['orderby'];
                } break;
                case 'date-desc': {
                    $clauses['orderby'] = " $wpdb->posts.post_date DESC, " . $clauses['orderby'];
                } break;
                case 'mod-asc': {
                    $clauses['orderby'] = " $wpdb->posts.post_modified ASC, " . $clauses['orderby'];
                } break;
                case 'mod-desc': {
                    $clauses['orderby'] = " $wpdb->posts.post_modified DESC, " . $clauses['orderby'];
                } break;
                case 'author-asc': {
                    $clauses['orderby'] = " $wpdb->posts.post_author ASC, " . $clauses['orderby'];
                } break;
                case 'author-desc': {
                    $clauses['orderby'] = " $wpdb->posts.post_author DESC, " . $clauses['orderby'];
                } break;
            }
        }
        return $clauses;
    }

    private function flat_to_tree(array &$folders, $parent = 0): array {
        $branch = [];
        foreach($folders as $key => $folder) {
            if ($folder['parent'] == $parent) {
                $children = $this->flat_to_tree($folders, $folder['id']);

                if($children) {
                    $folder['items'] = $children;
                }

                if(array_key_exists('items', $folder) && $folder['items']) {
                    $branch[] = [
                        'id' => $folder['id'],
                        'title' => $folder['title'],
                        'items' => $folder['items'],
                        'color' => $folder['color'],
                        'count' => intval($folder['count'], 10)
                    ];
                } else {
                    $branch[] = [
                        'id' => $folder['id'],
                        'title' => $folder['title'],
                        'color' => $folder['color'],
                        'count' => intval($folder['count'], 10)
                    ];
                }
                unset($folders[$key]);
            }
        }
        return $branch;
    }

    private function get_child_folders($parent, &$out) {
        global $wpdb;
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

        $sql = $wpdb->prepare("SELECT id FROM {$tableFolders} WHERE parent=%d", $parent);
        $items = $wpdb->get_results($sql, 'ARRAY_A');

        if($items) {
            foreach($items as $item) {
                $out[] = $item['id'];
                $this->get_child_folders($item['id'], $out);
            }
        }
    }

    private function get_token() {
        global $wp_version;
        $current_user = wp_get_current_user();

        $data = [
            'plugin_name' => WPCOMMANDER_PLUGIN_NAME,
            'plugin_version' => WPCOMMANDER_PLUGIN_VERSION,
            'wordpress' => $wp_version,
            'php' => PHP_VERSION,
            'email' => $current_user->user_email,
            'site' => trim(str_replace(['http://', 'https://'], '', get_site_url()), '/')
        ];
        return base64_encode(json_encode($data));
    }

    /**
     * Ajax methods
     */
    public function ajax_get_template() {
        $data = null;
        $result = false;

        if($this->has_access() && check_ajax_referer('wpcommander-ajax-nonce', 'nonce', false)) {
            $template = sanitize_key(filter_input(INPUT_POST, 'template', FILTER_DEFAULT));
            $file = plugin_dir_path(dirname(__FILE__)) . 'templates/' . $template . '.php';

            if(file_exists($file)) {
                ob_start(); // turn on buffering
                require_once($file);
                $data = ob_get_contents(); // get the buffered content into a var
                ob_end_clean(); // clean buffer
                $result = true;
            }
        }

        $result ? wp_send_json_success($data) : wp_send_json_error($data);
        wp_die(); // this is required to terminate immediately and return a proper response
    }

    public function ajax_get_app_data() {
        $data = null;
        $result = false;

        if($this->has_access() && check_ajax_referer('wpcommander-ajax-nonce', 'nonce', false)) {
            $inputData = filter_input(INPUT_POST, 'data', FILTER_DEFAULT);
            $jsonData = json_decode($inputData);

            $user_id = get_current_user_id();

            switch ($jsonData->target) {
                case 'meta': {
                    $meta = $this->get_meta($jsonData->type);

                    if($meta) {
                        $data = $meta;
                        $result = true;
                    }
                } break;
                case 'folders': {
                    global $wpdb;
                    $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

                    $sql = $wpdb->prepare("
                        SELECT id, title, parent, color, count 
                        FROM {$tableFolders} 
                        WHERE user_id=%d
                        ORDER BY sort, created",
                        $user_id
                    );
                    $folders = $wpdb->get_results($sql, 'ARRAY_A');

                    if (!$wpdb->last_error) {
                        $data['folders'] = $this->flat_to_tree($folders);
                        $result = true;
                    }
                } break;
                case 'attach_count': {
                    global $wpdb;
                    $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
                    $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

                    // folders to refresh after the update
                    $folders = [];
                    if(property_exists($jsonData, 'folders') && $jsonData->folders) {
                        $ids = implode(',', array_map('absint', $jsonData->folders));
                        $sql = $wpdb->prepare("
                            SELECT id, count 
                            FROM {$tableFolders} 
                            WHERE id IN(%1s)",
                            $ids
                        );
                        $folders = $wpdb->get_results($sql, 'ARRAY_A');

                        foreach ($folders as $key => $folder) {
                            $folders[$key]['count'] = intval($folder['count'], 10);
                        }
                    }

                    $sql = $wpdb->prepare("
                        SELECT COUNT(id) 
                        FROM $wpdb->posts 
                        WHERE post_type=%s AND post_status!=%s",
                        $jsonData->type, 'auto-draft'
                    );
                    $count = intval($wpdb->get_var($sql), 10);
                    $folders[] = ['id' => '-1', 'count' => $count]; // All files

                    $sql = $wpdb->prepare("
                        SELECT COUNT(id) 
                        FROM $wpdb->posts 
                        WHERE post_type=%s AND post_status!=%s AND id NOT IN (SELECT attachment_id FROM $tableAttachments AS A LEFT JOIN $tableFolders AS F ON F.id = A.folder_id WHERE F.user_id=%d)",
                        $jsonData->type, 'auto-draft', $user_id
                    );
                    $count = intval($wpdb->get_var($sql), 10);
                    $folders[] = array('id' => '-2', 'count' => $count); // Uncategorized

                    if(!$wpdb->last_error) {
                        $data['folders'] = $folders;
                        $result = true;
                    }
                } break;
                case 'contextmenu': {
                    $data = [
                        [
                            'id' => 'color',
                            'right' => 'e',
                            'title' => esc_html__('Color', 'wpcommander'),
                            'icon' => '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d=" M 1.313 12 L 9.197 19.875 L 18.197 10.875 L 10.322 3 L 1.313 12 L 1.313 12 Z  M 6.384 9.75 L 10.316 5.677 L 14.298 9.75 L 6.384 9.75 Z  M 19.312 12 C 21.562 14.508 22.687 16.382 22.687 17.625 C 22.687 19.489 21.176 21 19.312 21 C 17.449 21 15.937 19.489 15.937 17.625 C 15.937 16.382 17.063 14.508 19.312 12 L 19.312 12 Z " /></svg>'
                        ],
                        [
                            'id' => 'rename',
                            'right' => 'e',
                            'title' => esc_html__('Rename', 'wpcommander'),
                            'icon' => '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d=" M 6.774 8.994 L 4.887 12.768 C 4.868 12.8 4.851 12.833 4.837 12.868 L 4.084 14.373 C 3.991 14.56 3.975 14.77 4.037 14.972 C 4.107 15.167 4.247 15.33 4.426 15.416 C 4.613 15.509 4.823 15.525 5.026 15.463 C 5.22 15.393 5.384 15.253 5.469 15.074 L 6.033 13.945 L 8.967 13.945 L 8.967 13.945 L 9.531 15.074 C 9.616 15.253 9.78 15.393 9.974 15.463 C 10.177 15.525 10.387 15.509 10.574 15.416 C 10.753 15.33 10.893 15.167 10.963 14.972 C 11.025 14.77 11.009 14.56 10.916 14.373 L 8.193 8.926 C 8.107 8.747 7.944 8.607 7.749 8.537 C 7.547 8.475 7.337 8.491 7.15 8.584 C 6.981 8.665 6.847 8.814 6.774 8.994 Z  M 8.189 12.389 L 7.5 11.012 L 6.811 12.389 L 8.189 12.389 L 8.189 12.389 L 8.189 12.389 Z " fill-rule="evenodd" /><path d=" M 14 7.004 L 21 7 L 21 7 L 21 17 L 14 17.004 L 14 7.004 L 14 7.004 Z  M 12 7 L 3.098 7 L 3 17 L 12 17 L 12 7 L 12 7 L 12 7 Z  M 12 19 L 1.786 19 L 1.786 19 C 1.352 19 1 18.648 1 18.214 L 1 5.795 C 1 5.361 1.352 5.009 1.786 5.01 L 12 5.005 L 12 2.996 L 11 2.996 L 11 1.996 L 15 2 L 15 2.996 L 14 2.996 L 14 5.004 L 22.214 5 C 22.648 5 23 5.352 23 5.786 L 23 18.214 C 23 18.648 22.648 19 22.214 19 L 14 19.004 L 14 21 L 15 21 L 15 22 L 14 22 L 14 22 L 12 21.996 L 12 21.996 L 11 21.996 L 11 20.996 L 12 20.996 L 12 19 Z " fill-rule="evenodd" /></svg>'
                        ],
                        [
                            'id' => 'delete',
                            'right' => 'd',
                            'title' => esc_html__('Delete', 'wpcommander'),
                            'icon' => '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d=" M 10.423 3 C 10.01 3 9.588 3.145 9.289 3.444 C 8.99 3.742 8.846 4.164 8.846 4.577 L 8.846 5.366 L 4.114 5.366 L 4.114 6.943 L 4.902 6.943 L 4.902 19.561 C 4.902 20.858 5.971 21.927 7.268 21.927 L 16.732 21.927 C 18.029 21.927 19.098 20.858 19.098 19.561 L 19.098 6.943 L 19.886 6.943 L 19.886 5.366 L 15.154 5.366 L 15.154 4.577 C 15.154 4.164 15.01 3.742 14.711 3.444 C 14.412 3.145 13.99 3 13.577 3 L 10.423 3 Z  M 10.423 4.577 L 13.577 4.577 L 13.577 5.366 L 10.423 5.366 L 10.423 4.577 Z  M 6.48 6.943 L 17.52 6.943 L 17.52 19.561 C 17.52 19.998 17.169 20.35 16.732 20.35 L 7.268 20.35 C 6.831 20.35 6.48 19.998 6.48 19.561 L 6.48 6.943 Z  M 8.057 9.309 L 8.057 17.984 L 9.634 17.984 L 9.634 9.309 L 8.057 9.309 Z  M 11.211 9.309 L 11.211 17.984 L 12.789 17.984 L 12.789 9.309 L 11.211 9.309 Z  M 14.366 9.309 L 14.366 17.984 L 15.943 17.984 L 15.943 9.309 L 14.366 9.309 Z "/></svg>'
                        ]
                    ];
                    $result = true;
                } break;
            }
        }

        $result ? wp_send_json_success($data) : wp_send_json_error($data);
        wp_die(); // this is required to terminate immediately and return a proper response
    }

    public function ajax_update_app_data() {
        $data = null;
        $result = false;

        if($this->has_access() && check_ajax_referer('wpcommander-ajax-nonce', 'nonce', false)) {
            $inputData = filter_input(INPUT_POST, 'data', FILTER_DEFAULT);
            $jsonData = json_decode($inputData);

            $user_id = get_current_user_id();

            switch($jsonData->target) {
                    case 'folders': {
                        global $wpdb;
                        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
                        $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

                        switch($jsonData->action) {
                            case 'create': {
                                $folders = [];

                                foreach ($jsonData->names as $name) {
                                    $wpdb->insert(
                                        $tableFolders,
                                        [
                                            'title' => sanitize_text_field($name),
                                            'parent' => $jsonData->parent,
                                            'color' => $jsonData->color,
                                            'user_id' => $user_id,
                                            'created' => current_time('mysql', 1),
                                            'modified' => current_time('mysql', 1),
                                            'sort' => PHP_INT_MAX
                                        ]
                                    );

                                    if (!$wpdb->last_error) {
                                        $folders[] = ['id' => strval($wpdb->insert_id), 'title' => sanitize_text_field($name), 'color' => $jsonData->color];
                                    } else {
                                        break;
                                    }
                                }

                                if (!$wpdb->last_error) {
                                    $data['folders'] = $folders;
                                    $result = true;
                                }
                            } break;
                            case 'rename': {
                                $sql = $wpdb->prepare("
                                    UPDATE {$tableFolders} 
                                    SET title=%s, modified=%s 
                                    WHERE id=%d AND user_id=%d",
                                    sanitize_text_field($jsonData->name), current_time('mysql', 1), $jsonData->folder,  $user_id
                                );
                                $wpdb->query($sql); // $rows = $wpdb->query($sql); UPDATE will return 0 if a field value is equal to a new value

                                if(!$wpdb->last_error) {
                                    $data['folder'] = $jsonData->folder;
                                    $data['name'] = sanitize_text_field($jsonData->name);
                                    $result = true;
                                }

                            } break;
                            case 'color': {
                                $ids = implode(',', array_map('absint', $jsonData->folders));
                                $sql = $wpdb->prepare("
                                    SELECT id 
                                    FROM {$tableFolders} 
                                    WHERE id IN(%1s) AND user_id=%d",
                                    $ids,  $user_id
                                );
                                $folders = $wpdb->get_results($sql, 'ARRAY_A');

                                $folders_to_color = [];
                                foreach($folders as $folder) {
                                    $folders_to_color[] = $folder['id'];
                                }

                                if(!empty($folders_to_color)) {
                                    $ids = implode(',', array_map('absint', $folders_to_color));
                                    if($jsonData->color) {
                                        $sql = $wpdb->prepare("
                                            UPDATE {$tableFolders} 
                                            SET color=%s, modified=%s 
                                            WHERE id IN(%1s)",
                                            $jsonData->color, current_time('mysql', 1), $ids
                                        );
                                    } else {
                                        $sql = $wpdb->prepare("
                                            UPDATE {$tableFolders} 
                                            SET color=NULL, modified=%s 
                                            WHERE id IN(%1s)",
                                            current_time('mysql', 1), $ids
                                        );
                                    }
                                    $wpdb->query($sql);

                                    if(!$wpdb->last_error) {
                                        $data['folders'] = $folders_to_color;
                                        $data['color'] = $jsonData->color;
                                        $result = true;
                                    }
                                }
                            } break;
                            case 'move': {
                                $folder_parent = $jsonData->parent;
                                if($folder_parent != 0) {
                                    $sql = $wpdb->prepare("
                                        SELECT id 
                                        FROM {$tableFolders} 
                                        WHERE id=%d AND user_id=%d",
                                        $folder_parent, $user_id
                                    );
                                    $folder_parent = $wpdb->get_var($sql);
                                }

                                $ids = implode(',', array_map('absint', $jsonData->folders));
                                $sql = $wpdb->prepare("
                                    SELECT id 
                                    FROM {$tableFolders} 
                                    WHERE id IN(%1s) AND user_id=%d",
                                    $ids, $user_id
                                );
                                $folders = $wpdb->get_results($sql, 'ARRAY_A');

                                $folders_to_move = [];
                                foreach ($folders as $folder) {
                                    $folders_to_move[] = $folder['id'];
                                }

                                if(!empty($folders_to_move) && isset($folder_parent)) {
                                    $ids = implode(',', array_map('absint', $folders_to_move));

                                    if(isset($user_id_parent)) {
                                        $sql = $wpdb->prepare("
                                            UPDATE {$tableFolders}
                                            SET parent=%d, user_id=%d
                                            WHERE id IN(%1s)",
                                            $folder_parent, $user_id_parent, $ids
                                        );
                                    } else {
                                        $sql = $wpdb->prepare("
                                            UPDATE {$tableFolders} 
                                            SET parent=%d 
                                            WHERE id IN(%1s)",
                                            $folder_parent, $ids
                                        );
                                    }
                                    $wpdb->query($sql);

                                    $sort = 0; //!!!
                                    foreach ($jsonData->sorting as $id) {
                                        $sql = $wpdb->prepare("
                                            UPDATE {$tableFolders} 
                                            SET sort=%d 
                                            WHERE id=%d",
                                            $sort++, $id
                                        );
                                        $wpdb->query($sql);
                                    }

                                    if(!$wpdb->last_error) {
                                        unset($data['msg']);
                                        $data['folders'] = $folders_to_move;
                                        $data['parent'] = $folder_parent;
                                        $result = true;
                                    }
                                }
                            } break;
                            case 'delete': {
                                $ids = implode(',', array_map('absint', $jsonData->folders));
                                $sql = $wpdb->prepare("
                                    SELECT id 
                                    FROM {$tableFolders} 
                                    WHERE id IN(%1s) AND user_id=%d",
                                    $ids, $user_id
                                );
                                $folders = $wpdb->get_results($sql, 'ARRAY_A');

                                $folders_to_delete = [];
                                foreach ($folders as $folder) {
                                    $folders_to_delete[] = $folder['id'];
                                }

                                if(!empty($folders_to_delete)) {
                                    $folders = [];
                                    foreach ($folders_to_delete as $id) {
                                        $folders[] = $id;
                                        $this->get_child_folders($id, $folders);
                                    }
                                    $folders = array_values(array_unique($folders));

                                    $ids = implode(',', array_map('absint', $folders));
                                    $sql = $wpdb->prepare("DELETE FROM {$tableAttachments} WHERE folder_id IN(%1s)", $ids);
                                    $wpdb->query($sql);

                                    $sql = $wpdb->prepare("DELETE FROM {$tableFolders} WHERE id IN(%1s)", $ids);
                                    $wpdb->query($sql);

                                    if(!$wpdb->last_error) {
                                        $data['folders'] = $folders;
                                        $result = true;
                                    }
                                }
                            } break;
                            case 'attach': {
                                $folder_dest = $jsonData->dest;
                                if(!($folder_dest == -1 || $folder_dest == -2)) {
                                    $sql = $wpdb->prepare("
                                        SELECT id 
                                        FROM {$tableFolders} 
                                        WHERE id=%d AND user_id=%d",
                                        $folder_dest, $user_id
                                    );
                                    $folder_dest = $wpdb->get_var($sql);
                                }

                                if(isset($folder_dest)) {
                                    // folders to refresh after the update
                                    $ids = implode(',', array_map('absint', $jsonData->attachments));
                                    $sql = $wpdb->prepare("
                                        SELECT DISTINCT A.folder_id as id 
                                        FROM {$tableAttachments} AS A 
                                        LEFT JOIN {$tableFolders} AS F 
                                        ON F.id=A.folder_id AND F.user_id=%d
                                        WHERE A.attachment_id IN(%1s)",
                                        $user_id, $ids
                                    );
                                    $folders = $wpdb->get_results($sql, 'ARRAY_A');

                                    $folders_to_refresh = array($folder_dest);
                                    foreach($folders as $folder) {
                                        if($folder['id'] != $folder_dest) {
                                            $folders_to_refresh[] = $folder['id'];
                                        }
                                    }

                                    // delete previous attachments
                                    $sql = $wpdb->prepare("
                                        DELETE A.* 
                                        FROM {$tableAttachments} AS A 
                                        LEFT JOIN {$tableFolders} AS F 
                                        ON F.id=A.folder_id AND F.user_id=%s
                                        WHERE A.attachment_id IN(%1s)",
                                        $user_id, $ids
                                    );
                                    $wpdb->query($sql);

                                    // add new attachments
                                    if(!($folder_dest == -1 || $folder_dest == -2)) {
                                        foreach ($jsonData->attachments as $attachment) {
                                            $wpdb->insert(
                                                $tableAttachments,
                                                [
                                                    'folder_id' => $folder_dest,
                                                    'attachment_id' => $attachment
                                                ]
                                            );
                                        }
                                    }

                                    // update the attachment count
                                    $ids = implode(',', array_map('absint', $folders_to_refresh));
                                    $sql = $wpdb->prepare("
                                        UPDATE {$tableFolders} AS F 
                                        SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id=F.id) 
                                        WHERE id IN(%1s)",
                                        $ids
                                    );
                                    $wpdb->query($sql);

                                    if(!$wpdb->last_error) {
                                        unset($data['msg']);
                                        $data['folders'] = $folders_to_refresh;
                                        $data['attachments'] = $jsonData->attachments;
                                        $result = true;
                                    }
                                }
                            } break;
                        }
                    } break;
                    case 'meta': {
                        $meta['folder'] = $jsonData->meta->folder;
                        $meta['collapsed'] = $jsonData->meta->collapsed;
                        $meta['sort'] = [
                            'items' => $jsonData->meta->sort->items
                        ];

                        $user_meta = (array)get_user_meta($user_id, 'wpcommander_states', true);
                        $user_meta['types'][$jsonData->type] = $meta;
                        update_user_meta($user_id, 'wpcommander_states', $user_meta);

                        $result = true;
                    } break;
                }
        }

        $result ? wp_send_json_success($data) : wp_send_json_error($data);
        wp_die(); // this is required to terminate immediately and return a proper response
    }

    public function ajax_get_settings_data() {
        $data = null;
        $result = false;

        if(check_ajax_referer('wpcommander-ajax-nonce', 'nonce', false)) {
            $inputData = filter_input(INPUT_POST, 'data', FILTER_DEFAULT);
            $jsonData = json_decode($inputData);

            switch($jsonData->target) {
                case 'folders': {
                    global $wpdb;
                    $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
                    $tableUsers = $wpdb->users;

                    switch($jsonData->type) {
                        case 'total': {
                            $sql = "SELECT COUNT(*) as total FROM {$tableFolders}";
                            $total = $wpdb->get_var($sql);

                            if(!$wpdb->last_error) {
                                $data['total'] = intval($total, 10);
                                $result = true;
                            }
                        } break;
                        case 'list': {
                            $page = intval($jsonData->page, 10);
                            $count = intval($jsonData->itemsperpage, 10);
                            $columns = array('title', 'owner', 'attachments', 'created', 'modified');
                            $order_column = in_array($jsonData->order->column, $columns) ? $jsonData->order->column : 'modified';
                            $orders = array('asc', 'desc');
                            $order_type = in_array($jsonData->order->type, $orders) ? $jsonData->order->type : 'desc';
                            $offset = ($page - 1) * $count;


                            $sql = "SELECT COUNT(*) as total FROM {$tableFolders}";
                            $total = $wpdb->get_var($sql);

                            $sql = $wpdb->prepare("
                                SELECT F.id, F.title, F.color, U.display_name AS owner, F.count as attachments, F.created, F.modified 
                                FROM {$tableFolders} AS F
                                LEFT JOIN {$tableUsers} AS U ON U.id = F.user_id
                                ORDER BY {$order_column} {$order_type} 
                                LIMIT %d, %d",
                                $offset, $count
                            );
                            $list = $wpdb->get_results($sql, 'ARRAY_A');

                            if(!$wpdb->last_error) {
                                foreach($list as $key => $item) {
                                    $list[$key]['attachments'] = intval($item['attachments'], 10);
                                    $list[$key]['created'] = date("d.m.Y", strtotime($item['created']));
                                    $list[$key]['modified'] = date("d.m.Y", strtotime($item['modified']));
                                }

                                $data['test'] = $order_column;
                                $data['total'] = intval($total, 10);
                                $data['list'] = $list;
                                $result = true;
                            }
                        } break;
                    }
                } break;
                case 'roles': {
                    $data['list'] = array();
                    $result = true;

                    $roles = wp_roles()->roles;
                    foreach ($roles as $key => $role) {
                        if (array_key_exists('read', $role['capabilities'])) {
                            $data['list'][] = array('id' => $key, 'name' => translate_user_role($role['name']));
                        }
                    }
                } break;
                case 'config': {
                    $data['config'] = $this->get_config_data();
                    $result = true;
                } break;
            }
        }

        $result ? wp_send_json_success($data) : wp_send_json_error($data);
        wp_die(); // this is required to terminate immediately and return a proper response
    }

    public function ajax_update_settings_data() {
        $data = null;
        $result = false;

        if(check_ajax_referer('wpcommander-ajax-nonce', 'nonce', false)) {
            $inputData = filter_input(INPUT_POST, 'data', FILTER_DEFAULT);
            $jsonData = json_decode($inputData);

            switch($jsonData->target) {
                case 'folders': {
                    global $wpdb;
                    $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
                    $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

                    switch($jsonData->action) {
                        case 'delete': {
                            switch($jsonData->type) {
                                case 'list': {
                                    $folders = [];
                                    foreach($jsonData->list as $id) {
                                        $folders[] = $id;
                                        $this->get_child_folders($id, $folders);
                                    }
                                    $folders = array_values(array_unique($folders));

                                    $ids = implode(',', array_map('absint', $folders));
                                    $sql = $wpdb->prepare("DELETE FROM {$tableAttachments} WHERE folder_id IN(%1s)", $ids);
                                    $result = $wpdb->query($sql);

                                    $sql = $wpdb->prepare("DELETE FROM {$tableFolders} WHERE id IN(%1s)", $ids);
                                    $result = $wpdb->query($sql);

                                    if($result !== false) {
                                        $data['msg'] = esc_html__('The operation completed successfully', 'wpcommander');
                                        $result = true;
                                    } else {
                                        $data['msg'] = esc_html__('The operation failed', 'wpcommander');
                                        $result = false;
                                    }
                                } break;
                                case 'all': {
                                    $sql = "DELETE FROM {$tableAttachments}";
                                    $result = $wpdb->query($sql);

                                    $sql = "DELETE FROM {$tableFolders}";
                                    $result = $wpdb->query($sql);

                                    if($result !== false) {
                                        $data['msg'] = esc_html__('The operation completed successfully', 'wpcommander');
                                        $result = true;
                                    } else {
                                        $data['msg'] = esc_html__('The operation failed', 'wpcommander');
                                        $result = false;
                                    }
                                } break;
                            }
                        } break;
                    }
                } break;
                case 'config': {
                    $config = $jsonData->config;
                    $result = $this->set_config_data($config);
                } break;
                case 'dismiss-first-use-notification': {
                    $result = update_option('wpcommander_dismiss_first_use_notification', true, false);
                } break;
            }
        }

        $result ? wp_send_json_success($data) : wp_send_json_error($data);
        wp_die(); // this is required to terminate immediately and return a proper response
    }
}